--[[
*************************************************************
 * This script is developed by Arturs Sosins aka ar2rsawseen, http://appcodingeasy.com
 * Feel free to distribute and modify code, but keep reference to its creator
 *
 * Gideros Game Template for developing games. Includes: 
 * Start scene, pack select, level select, settings, score system and much more
 *
 * For more information, examples and online documentation visit: 
 * http://appcodingeasy.com/Gideros-Mobile/Gideros-Mobile-Game-Template
**************************************************************
]]--

level = gideros.class(Sprite)

function level:init()
 self:addEventListener("enterEnd", self.onEnterEnd, self)
 self:addEventListener("exitBegin", self.onExitBegin, self)
 self:addEventListener("enterBegin", self.onEnterBegin, self)
 self:addEventListener("exitEnd", self.onExitEnd, self)
end

--[[enterBegin: Dispatched when your new scene is about to enter the stage. You can initialize your variables here.
enterEnd: Dispatched after the transition of new scene has ended. Mostly, you add listeners and start your logic here.
exitBegin: Dispatched when your current scene is about to leave the stage. You can remove listeners and stop timers here.
exitEnd: Dispatched after your current scene has leaved the stage.]]

function level:onEnterBegin()

--Dispatched when your new scene is about to enter the stage. You can initialize your variables here.
	accelerometer:start()
	
	--Load tilemap
	map = TileMapMultiple.new("Levels/level-"..sets.curPack.."-"..sets.curLevel..".lua")
	--add tilemap to level
	self:addChild(map)
	-- add player to the level
	player = Player.new()
	self:addChild(player)
	
	--use player and map to create camera
	self.camera = Camera.new(map, player)
	
	--load scores
	local highScore = dataSaver.loadValue("scores")
	--if not exist yet
	if(not highScore) then
		highScore = {}
	end
	--score for this level
	if(not highScore[(sets.curPack).."-"..sets.curLevel]) then
		highScore[(sets.curPack).."-"..sets.curLevel] = {}
		highScore[(sets.curPack).."-"..sets.curLevel].score = 0
	end
	currentScore = 0

	pauseGame = false
	
	--display highscore
	local highscore = TextField.new(nil, "Highscore: "..highScore[(sets.curPack).."-"..sets.curLevel].score)
	highscore:setPosition(10,application:getContentHeight()-30)
	highscore:setTextColor(0x000000)
	self:addChild(highscore)
	
	--display current score
	self.score = TextField.new(nil, "Score: 0")
	self.score:setPosition(10,application:getContentHeight()-20)
	self.score:setTextColor(0x000000)
	self:addChild(self.score)
	
	--reset button
	resetButton = Button.new(Bitmap.new(Texture.new("Assets/grfx/reset_up.png")), Bitmap.new(Texture.new("Assets/grfx/reset_down.png")))
	resetButton:setPosition(20, 10)
	self:addChild(resetButton)
	resetButton:addEventListener("click", 
		function()	
			sceneManager:changeScene("level", 1, transition, easing.outBack)
		end)

end


function level:onEnterEnd()
--Dispatched after the transition of new scene has ended. Mostly, you add listeners and start your logic here.

	self:addEventListener(Event.ENTER_FRAME, self.onEnterFrame)
	self:addEventListener(Event.MOUSE_DOWN, self.onMouseDown, self)
	
--start accelerometer
	accelerometer:start()
	
end

--removing event
function level:onEnterFrame() 
	if not startGame and not pauseGame then
		-- update frame functions here
	end
end

--removing event on exiting scene
function level:onExitBegin()
	self.camera = nil
	self:removeChild(map)
	self:removeChild(player)
-- remove listeners and stop timers here.
  accelerometer:stop()
  self:removeEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
end

function level:onExitEnd()


end

function level:onMouseDown( event )

	
end

function level:saveScore()
		if(currentScore > highScore[(sets.curPack).."-"..sets.curLevel].score) then
			highScore[(sets.curPack).."-"..sets.curLevel].score = currentScore
		end
		--increase level
		sets.curLevel = sets.curLevel + 1
		
		--if level does not exist in pack
		if(packs.packs[sets.curPack].levels < sets.curLevel) then
			--level one
			sets.curLevel = 1
			--increase pack
			sets.curPack = sets.curPack + 1
			--if pack exists
			if(packs.packs[sets.curPack]) then
				--unlock pack
				local unPacks = dataSaver.loadValue("unPacks")
				if(not unPacks) then
					unPacks = {}
				end
				unPacks[sets.curPack] = true
				dataSaver.saveValue("unPacks", unPacks)
			else
				--if doesn't exist, go back
				sets.curPack = sets.curPack - 1
			end
			--unlock next level
			if(not highScore[(sets.curPack).."-"..sets.curLevel]) then
				highScore[(sets.curPack).."-"..sets.curLevel] = {}
				highScore[(sets.curPack).."-"..sets.curLevel].score = 0
				highScore[(sets.curPack).."-"..sets.curLevel].unlocked = true
			end
			--save high score
			dataSaver.saveValue("scores", highScore)
			ret = "pack_select"
		else
			--unlock next level
			if(not highScore[(sets.curPack).."-"..sets.curLevel]) then
				highScore[(sets.curPack).."-"..sets.curLevel] = {}
				highScore[(sets.curPack).."-"..sets.curLevel].score = 0
				highScore[(sets.curPack).."-"..sets.curLevel].unlocked = true
			end
			--save high score
			dataSaver.saveValue("scores", highScore)
			ret = "level"
		end
		--save setting
		dataSaver.saveValue("sets", sets)
		--return to scene
		return ret
	end