TileMapMultiple = Core.class(Sprite)

local map

local function gid2tileset(map, gid)
	for i=1, #map.tilesets do
		local tileset = map.tilesets[i]
	
		if tileset.firstgid <= gid and gid <= tileset.lastgid then
			return tileset
		end
	end
end

function TileMapMultiple:init(filename)
	map = loadfile(filename)()
	
	--set map properties from lua file
	self.width = map.width*map.tilewidth
	self.height = map.height*map.tileheight

	for i=1, #map.tilesets do
		local tileset = map.tilesets[i]
		
		tileset.sizex = math.floor((tileset.imagewidth - tileset.margin + tileset.spacing) / (tileset.tilewidth + tileset.spacing))
		tileset.sizey = math.floor((tileset.imageheight - tileset.margin + tileset.spacing) / (tileset.tileheight + tileset.spacing))
		tileset.lastgid = tileset.firstgid + (tileset.sizex * tileset.sizey) - 1

		tileset.texture = Texture.new(tileset.image)
	end

	for i=1, #map.layers do
	
		local layer = map.layers[i]

		local tilemaps = {}
		local group = Sprite.new()
		--if layer.type == "tilelayer" then -- create layer only if tiles.
			for y=1, layer.height do
				for x=1,layer.width do
					local i = x + (y - 1) * layer.width
					local gid = layer.data[i]
					local tileset = gid2tileset(map, gid)
				
					if tileset then
						local tilemap = nil
						if tilemaps[tileset] then
							tilemap = tilemaps[tileset]
						else
							tilemap = TileMap.new(layer.width, 
												layer.height,
												tileset.texture,
												tileset.tilewidth,
												tileset.tileheight,
												tileset.spacing,
												tileset.spacing,
												tileset.margin,
												tileset.margin,
												map.tilewidth,
												map.tileheight)
							tilemaps[tileset] = tilemap
							group:addChild(tilemap)
						end
					
						local tx = (gid - tileset.firstgid) % tileset.sizex + 1
						local ty = math.floor((gid - tileset.firstgid) / tileset.sizex) + 1
					
						tilemap:setTile(x, y, tx, ty)
					end
				end
			end -- end for
		--end -- end if
		group:setAlpha(layer.opacity)
		
		self:addChild(group)
	end
end

function TileMapMultiple:getLayer(name)

	-- if we have a name parameter and it's a string
	if name and type(name) == 'string' then
		-- search for name
		for index = 1, #map.layers do
			-- if found
			if map.layers[index].name == name then
				-- return it
				return map.layers[index]
				
			end
			
		end
		
	end

end

function TileMapMultiple:getProperty(item, name)
	
	-- if we have an item and 
	if item and name and type(item) == 'table' and type(name) == 'string' then
		
		for k, v in pairs(item.properties) do 
			
			if k == name then  
				
				return v
				
			end	
			
		end
		
	end
	
end

--finds the block that the given x and y are in.
--returns x and y of block
function TileMapMultiple:findBlock(x,y)

	local oX, oY = self:getPosition()
	local X, Y = x, y
	local dX = X-oX -- the distance from the map origin x-axis
	local dY = Y - oY --  " 	" 	 " y-axis
	
	local width = map.tilewidth
	local height = map.tileheight
	
	local tileX = math.ceil(dX/width)
	local tileY = math.ceil(dY/height)
	
	return tileX, tileY
end

--returns the coordinates in pixels of the block
function TileMapMultiple:findPoints(x,y)

	local oX, oY = self:getPosition()
	local X, Y = x, y
	local dX = X-oX -- the distance from the map origin x-axis
	local dY = Y - oY --  " 	" 	 " y-axis
	
	local width = map.tilewidth
	local height = map.tileheight
	
	local pX = (X -1)*width + oX
	local pY = (Y -1)*height + oY
	return pX, pY
end

--finds the index in the layer.data array based on block coordinates
--returns the gid value of layer.data[index]
function TileMapMultiple:findData(x, y, layer)
	local Layer = layer
	local X, Y = x, y

	local width = map.width
	local height = map.height
	
	local index = (Y - 1)*width + X
	
	return Layer.data[index]
end

function TileMapMultiple:checkCollisionX(player, layer)

	local oX, oY = player:getPosition()
	local pWidth = player:getWidth()
	local pHeight = player:getHeight()
	
	local points = player:getCollisionPoints()
	local dX = player.newX -- newX and newY are set by player movement
	local dY = player.newY
	local bX = 0 --coordinates for blocks
	local bY = 0
	local index -- index of layer.data at block x - y
	local nX, nY -- new coordinates
	
	if dX > 0 then -- player is moving to right... check TR BR for collision
		bX, bY = self:findBlock(points.TR.x+dX,points.TR.y+10)	-- returns block at TR
		nX, nY = self:findPoints(bX, bY)
		index =  self:findData(bX, bY, layer)
		if index and index > 0 then -- it is not an empty square, so Collision is true
			player:setX(nX - pWidth)
			return
		end
		
		--repeat for BR ...
		bX, bY = self:findBlock(points.BR.x+dX,points.BR.y - 10)	-- returns block 
		nX, nY = self:findPoints(bX, bY)
		index =  self:findData(bX, bY, layer)
		if index and index > 0 then -- it is not an empty square, so Collision is true
			player:setX(nX - pWidth)
			return
		end
		
	elseif dX < 0 then --player is moving left .... check TL and BL
		print(player:getPosition())
		bX, bY = self:findBlock(points.TL.x+dX,points.TL.y+10)	-- returns block at TL
		nX, nY =self:findPoints(bX, bY)
		index =  self:findData(bX, bY, layer)
		if index and index > 0 then -- it is not an empty square, so Collision is true
			player:setX(nX + map.tilewidth)
			return
		end
		
		--repeat for BL ...
		bX, bY = self:findBlock(points.BL.x+dX,points.BL.y - 10)	-- returns block 
		nX, nY = self:findPoints(bX, bY)
		index =  self:findData(bX, bY, layer)
		if index and index > 0 then -- it is not an empty square, so Collision is true
			player:setX(nX + map.tilewidth)
			return
		end
	end
	-- if no collisions left or right, set new X
	player:setX(oX + dX)
	end
	
function TileMapMultiple:checkCollisionY(player, layer)

	local oX, oY = player:getPosition()
	local pWidth = player:getWidth()
	local pHeight = player:getHeight()
	
	local points = player:getCollisionPoints()
	local dX = player.newX -- newX and newY are set by player movement
	local dY = player.newY
	local bX = 0 --coordinates for blocks
	local bY = 0
	local index -- index of layer.data at block x - y
	local nX, nY -- new coordinates
	
	if dY < 0 then -- player is moving UP... check TR TL for collision
		bX, bY = self:findBlock(points.TR.x,points.TR.y+dY)	-- returns block at TR
		nX, nY = self:findPoints(bX, bY)
		index =  self:findData(bX, bY, layer)
		if index and index > 0 then -- it is not an empty square, so Collision is true
			player:setY(nY + map.tileheight)
			return
		end
		
		--repeat for TL ...
		bX, bY = self:findBlock(points.TL.x,points.TL.y+dY)	-- returns block 
		nX, nY = self:findPoints(bX, bY)
		index =  self:findData(bX, bY, layer)
		if index and index > 0 then -- it is not an empty square, so Collision is true
			player:setY(nY + map.tileheight)
			return
		end
		
	elseif dY > 0 then --player is moving DOWN .... check BL and BR
		bX, bY = self:findBlock(points.BL.x,points.BL.y+dY)	-- returns block at TR
		nX, nY =self:findPoints(bX, bY)
		index =  self:findData(bX, bY, layer)
		if index  and index > 0 then -- it is not an empty square, so Collision is true
			player:setY(nY - pHeight)
			player.onGround = true
			return
		end
		
		--repeat for BR ... PLAYER MOVING DOWN
		bX, bY = self:findBlock(points.BR.x,points.BL.y + dY)	-- returns block 
		nX, nY = self:findPoints(bX, bY)
		index =  self:findData(bX, bY, layer)
		if index and index > 0 then -- it is not an empty square, so Collision is true
			player:setY(nY - pHeight)
			player.onGround = true
			return
		end
	end
	-- if no collisions UP or DOWN, set new Y
	player:setY(oY + dY)
	player.onGround = false
	end